﻿using System.Collections.ObjectModel;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.Routing;
using System.Reflection;
using System.IO;

namespace CSharp_Silverlight_HowDoISamples.Services
{
    // NOTE: If you change the class name "ExtendingRoutingSource" here, you must also update the reference to "ExtendingRoutingSource" in Web.config.
    public class ExtendingRoutingSource : IExtendingRoutingSource
    {
        private static ShapeFileFeatureSource featureSource;
        private static RoutingSource routingSource;

        public ExtendingRoutingSource()
        {
            string excutePath = Path.GetDirectoryName(Assembly.GetAssembly(typeof(GeoDataGetting)).CodeBase);
            string appPath = excutePath.Substring(excutePath.IndexOf('\\') + 1, excutePath.LastIndexOf('\\') - excutePath.IndexOf('\\') - 1);
            featureSource = new ShapeFileFeatureSource(appPath + @"\App_Data\Austinstreets.shp");
            routingSource = new CustomRoutingSource(featureSource);
        }

        public string GetRoute(string startId, string endId)
        {
            RoutingEngine routingEngine = new RoutingEngine(routingSource, featureSource);

            RoutingResult result = routingEngine.GetRoute(startId, endId);
            string wkt = result.Route.GetWellKnownText();
            return wkt;
        }
    }

    /// 
    /// It shows how to exploler custom RoutingSource
    /// 
    public class CustomRoutingSource : RoutingSource
    {
        private FeatureSource featureSource;

        public CustomRoutingSource()
            : this(null)
        {
        }

        public CustomRoutingSource(FeatureSource featureSource)
            : base()
        {
            this.featureSource = featureSource;
        }

        protected override int GetRouteSegmentCountCore()
        {
            return featureSource.GetCount();
        }

        protected override void OpenCore()
        {
            base.OpenCore();
            featureSource.Open();
        }

        protected override void CloseCore()
        {
            base.CloseCore();
            featureSource.Close();
        }

        protected override RouteSegment GetRouteSegmentByFeatureIdCore(string roadId)
        {
            Feature feature = featureSource.GetFeatureById(roadId, ReturningColumnsType.NoColumns);

            LineBaseShape shape = feature.GetShape() as LineBaseShape;
            LineShape sourceShape = ConvertLineBaseShapeToLines(shape)[0];
            sourceShape.Id = feature.Id;
            Collection<string> startIds = new Collection<string>();
            Collection<string> endIds = new Collection<string>();
            GetAdjacentFeataureIds(featureSource, sourceShape, startIds, endIds);
            PointShape startPoint = new PointShape(sourceShape.Vertices[0]);
            PointShape endPoint = new PointShape(sourceShape.Vertices[sourceShape.Vertices.Count - 1]);
            float length = (float)sourceShape.GetLength(GeographyUnit.Meter, DistanceUnit.Meter);
            RouteSegment road = new RouteSegment(feature.Id, 0, length, startPoint, startIds, endPoint, endIds);

            return road;
        }

        private static void GetAdjacentFeataureIds(FeatureSource featureSource, LineShape sourceShape, Collection<string> startIds, Collection<string> endIds)
        {
            Vertex startVertex = sourceShape.Vertices[0];
            Vertex endVertex = sourceShape.Vertices[sourceShape.Vertices.Count - 1];
            Collection<Feature> tempfeatures = featureSource.GetFeaturesInsideBoundingBox(sourceShape.GetBoundingBox(), ReturningColumnsType.NoColumns);
            foreach (Feature tempFeature in tempfeatures)
            {
                LineShape tempShape = ((MultilineShape)tempFeature.GetShape()).Lines[0];
                if (sourceShape.Id == tempFeature.Id)
                {
                    continue;
                }

                if (tempShape.Vertices.Contains(startVertex))
                {
                    startIds.Add(tempFeature.Id);
                }
                else if (tempShape.Vertices.Contains(endVertex))
                {
                    endIds.Add(tempFeature.Id);
                }
            }
        }

        private static Collection<LineShape> ConvertLineBaseShapeToLines(LineBaseShape sourceShape)
        {
            Collection<LineShape> lines = null;
            LineShape lineShape = sourceShape as LineShape;
            if (lineShape != null)
            {
                lines = new Collection<LineShape>();
                lines.Add(lineShape);
            }
            else
            {
                lines = ((MultilineShape)sourceShape).Lines;
            }
            return lines;
        }
    }


}
