﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Globalization;
using System.Web.Services;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.Routing;
using System.IO;
using System.Reflection;
using System.Text;

namespace CSharp_Silverlight_HowDoISamples
{
    public class GeoDataGetting : IGeoDataGetting
    {
        private static object lockObject = new object();
        private static ShapeFileFeatureSource featureSource;
        private static RoutingEngine routingEngine;
        private static RoutingSource routingSource;

        private static RoutingEngine fastestRoutingEngine;
        private static RoutingSource fastRoutingSource;

        //public GeoDataGetting(string shapeFilePathName, string rtgFilePathName)
        public GeoDataGetting()
        {
            if (featureSource == null || routingSource == null)
            {
                string excutePath = Path.GetDirectoryName(Assembly.GetAssembly(typeof(GeoDataGetting)).CodeBase);
                string appPath = excutePath.Substring(excutePath.IndexOf('\\') + 1, excutePath.LastIndexOf('\\') - excutePath.IndexOf('\\') - 1);
                featureSource = new ShapeFileFeatureSource(appPath + @"\App_Data\Austinstreets.shp");
                routingSource = new RtgRoutingSource(appPath + @"\App_Data\Austinstreets.rtg");
                fastRoutingSource = new RtgRoutingSource(appPath + @"\App_Data\AustinStreetsForFastest.rtg");
            }
        }

        public GeoDataGetting(string shapeFilePathName, string rtgFilePathName)
        {
            if (featureSource == null || routingSource == null)
            {
                featureSource = new ShapeFileFeatureSource(shapeFilePathName);
                routingSource = new RtgRoutingSource(rtgFilePathName);
            }
        }

        [WebMethod]
        public string GetPointShapeById(string featureId)
        {
            featureSource.Open();
            return featureSource.GetFeatureById(featureId, ReturningColumnsType.NoColumns).GetShape().GetCenterPoint().ToString();
        }

        [WebMethod]
        public string GetRoute(string startFeatureId, string endFeatureId, string algorithm)
        {
            RoutingAlgorithm routingAlgorithm = new DijkstraRoutingAlgorithm();
            switch (algorithm)
            {
                case "A*":
                    routingAlgorithm = new AStarRoutingAlgorithm();
                    break;
                case "Bidirectional":
                    routingAlgorithm = new BidirectionalRoutingAlgorithm();
                    break;
                default:
                    break;
            }

            routingEngine = new RoutingEngine(routingSource, routingAlgorithm, featureSource);

            RoutingResult result = routingEngine.GetRoute(startFeatureId, endFeatureId);
            string wkt = result.Route.GetWellKnownText();
            return wkt;
        }

        [WebMethod]
        public string GetOneWayRoute(string startFeatureId, string endFeatureId)
        {
            string excutePath = Path.GetDirectoryName(Assembly.GetAssembly(typeof(GeoDataGetting)).CodeBase);
            string appPath = excutePath.Substring(excutePath.IndexOf('\\') + 1, excutePath.LastIndexOf('\\') - excutePath.IndexOf('\\') - 1);

            featureSource = new ShapeFileFeatureSource(appPath + @"\App_Data\AustinWithOneWayRoad.shp");
            routingSource = new RtgRoutingSource(appPath + @"\App_Data\AustinWithOneWayRoad.rtg");
            routingEngine = new RoutingEngine(routingSource, featureSource);

            RoutingResult result = routingEngine.GetRoute(startFeatureId, endFeatureId);
            string wkt = result.Route.GetWellKnownText();
            return wkt;
        }

        [WebMethod]
        public Dictionary<string, Collection<string>> GetRouteResult(string startFeatureId, string endFeatureId, string algorithm)
        {
            RoutingAlgorithm routingAlgorithm = new DijkstraRoutingAlgorithm();
            switch (algorithm)
            {
                case "A*":
                    routingAlgorithm = new AStarRoutingAlgorithm();
                    break;
                case "Bidirectional":
                    routingAlgorithm = new BidirectionalRoutingAlgorithm();
                    break;
                default:
                    break;
            }

            routingEngine = new RoutingEngine(routingSource, routingAlgorithm, featureSource);
            RoutingResult result = new RoutingResult();
            result = routingEngine.GetRoute(startFeatureId, endFeatureId);
            return SerialRoutingResult(result);
        }

        [WebMethod]
        public string GetFastestRoute(string startFeatureId, string endFeatureId)
        {
            if (fastestRoutingEngine == null)
            {
                fastestRoutingEngine = new RoutingEngine(fastRoutingSource, featureSource);
            }

            RoutingResult result = fastestRoutingEngine.GetRoute(startFeatureId, endFeatureId);
            string wkt = result.Route.GetWellKnownText();
            return wkt;
        }

        [WebMethod]
        public string GetRouteByCoordinate(string startPoint, string endPoint)
        {
            if (routingEngine == null)
            {
                routingEngine = new RoutingEngine(fastRoutingSource, featureSource);
            }

            lock (featureSource)
            {
                PointShape start = ConvertStringToPointShape(startPoint);
                PointShape end = ConvertStringToPointShape(endPoint);
                RoutingResult result = routingEngine.GetRoute(start, end);
                string wkt = result.Route.GetWellKnownText();
                return wkt;
            }
        }

        [WebMethod]
        public string GetServiceArea(string centerPoint, TimeSpan drivingTime, float speed, string speedUnit)
        {
            SpeedUnit unit = SpeedUnit.Kph;
            switch (speedUnit)
            {
                case "Mile":
                    unit = SpeedUnit.Mph;
                    break;
                default:
                    break;
            }
            PointShape source = ConvertStringToPointShape(centerPoint);
            if (routingEngine == null)
            {
                routingEngine = new RoutingEngine(routingSource, featureSource);
            }
            PolygonShape polygon = routingEngine.GenerateServiceArea(source, drivingTime, speed, unit);
            return polygon.GetWellKnownText();
        }

        [WebMethod]
        public Dictionary<string, string> GetRouteSegmentByFeatureId(string featureId)
        {
            routingSource.Open();
            ((RtgRoutingSource)routingSource).ReadEndPoints = true;
            RouteSegment segment = routingSource.GetRouteSegmentByFeatureId(featureId);
            return SerailRouteSegment(segment);
        }

        [WebMethod]
        public string GetFeatureById(string id)
        {
            featureSource.Open();
            Feature feature = featureSource.GetFeatureById(id, ReturningColumnsType.AllColumns);
            return feature.GetWellKnownText();
        }

        [WebMethod]
        public Collection<string> GetFeaturesByIds(IEnumerable<string> ids)
        {
            featureSource.Open();
            Collection<Feature> features = featureSource.GetFeaturesByIds(ids, ReturningColumnsType.NoColumns);
            Collection<string> wkts = new Collection<string>();
            foreach (Feature feature in features)
            {
                wkts.Add(feature.GetWellKnownText());
            }
            return wkts;
        }

        [WebMethod]
        public string GetCenterPointOnLine(string pointShape)
        {
            featureSource.Open();
            PointShape point = ConvertStringToPointShape(pointShape);
            Collection<Feature> features = featureSource.GetFeaturesNearestTo(point, GeographyUnit.DecimalDegree, 1, ReturningColumnsType.NoColumns);
            if (features.Count > 0)
            {
                LineBaseShape line = features[0].GetShape() as LineBaseShape;
                PointShape centerPoint = line.GetCenterPoint();
                return String.Format("{0},{1}", centerPoint.X.ToString(CultureInfo.InvariantCulture), centerPoint.Y.ToString(CultureInfo.InvariantCulture));
            }
            return string.Empty;
        }

        [WebMethod]
        public string GetRouteViaStops(string startPoint, Collection<string> stopPoints, int iterations)
        {
            if (routingEngine == null)
            {
                routingEngine = new RoutingEngine(routingSource, featureSource);
            }

            PointShape start = ConvertStringToPointShape(startPoint);
            Collection<PointShape> points = new Collection<PointShape>();
            foreach (string point in stopPoints)
            {
                points.Add(ConvertStringToPointShape(point));
            }

            RoutingResult result = routingEngine.GetRoute(start, points, iterations);
            string visitSequence = string.Empty;
            foreach (PointShape point in result.OrderedStops)
            {
                int order = points.IndexOf(point);
                visitSequence += order.ToString(CultureInfo.InvariantCulture) + "-";
            }
            string wkt = result.Route.GetWellKnownText() + "&" + visitSequence.TrimEnd('-');
            return wkt;
        }

        private PointShape ConvertStringToPointShape(string point)
        {
            string[] coordinates = point.Split(',');
            double x = double.Parse(coordinates[0], CultureInfo.InvariantCulture);
            double y = double.Parse(coordinates[1], CultureInfo.InvariantCulture);

            return new PointShape(x, y);
        }

        private Dictionary<string, string> SerailRouteSegment(RouteSegment segment)
        {
            Dictionary<string, string> values = new Dictionary<string, string>();
            values.Add("startPoint", segment.StartPoint.ToString());
            values.Add("endPoint", segment.EndPoint.ToString());
            values.Add("featureId", segment.FeatureId);
            values.Add("type", segment.RouteSegmentType.ToString(CultureInfo.InvariantCulture));
            string startIds = string.Empty;
            foreach (string id in segment.StartPointAdjacentIds)
            {
                startIds += id + "_";
            }
            values.Add("startAdjacentIds", startIds.TrimEnd('_'));
            string endIds = string.Empty;
            foreach (string id in segment.EndPointAdjacentIds)
            {
                endIds += id + "_";
            }
            values.Add("endAdjacentIds", endIds.TrimEnd('_'));

            return values;
        }

        private Dictionary<string, Collection<string>> SerialRoutingResult(RoutingResult result)
        {
            Dictionary<string, Collection<string>> values = new Dictionary<string, Collection<string>>();

            values["wkt"] = new Collection<string>() { result.Route.GetWellKnownText() };
            Collection<string> roadNames = new Collection<string>();
            Collection<string> roadWkt = new Collection<string>();
            foreach (Feature feature in result.Features)
            {
                roadNames.Add(feature.ColumnValues["FENAME"]);
                roadWkt.Add(feature.GetWellKnownText());
            }
            values["roadWkts"] = roadWkt;
            values["roadName"] = roadNames;
            Collection<string> directions = new Collection<string>();
            foreach (RouteSegment segment in result.RouteSegments)
            {
                directions.Add(segment.DrivingDirection.ToString());
            }
            values["direction"] = directions;

            return values;
        }
    }
}
