﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Web.Services;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.Routing;
using System.Reflection;
using System.IO;

namespace CSharp_Silverlight_HowDoISamples
{
    // NOTE: If you change the class name "GetRouteInArea" here, you must also update the reference to "GetRouteInArea" in Web.config.
    public class GetRouteInArea : IGetRouteInArea
    {
        private static ShapeFileFeatureSource featureSource;
        private static RoutingEngine routingEngine;
        private static RoutingSource routingSource;
        private static Collection<string> allowFeatureIds;

        public GetRouteInArea()
        {
            string excutePath = Path.GetDirectoryName(Assembly.GetAssembly(typeof(GeoDataGetting)).CodeBase);
            string appPath = excutePath.Substring(excutePath.IndexOf('\\') + 1, excutePath.LastIndexOf('\\') - excutePath.IndexOf('\\') - 1);

            featureSource = new ShapeFileFeatureSource(appPath + @"\App_Data\Austinstreets.shp");
            routingSource = new RtgRoutingSource(appPath + @"\App_Data\Austinstreets.rtg");
        }

        [WebMethod]
        public string GetRouteInSepecifiedArea(string startFeatureId, string endFeatureId, string avoidWkt)
        {
            if (routingEngine == null)
            {
                routingEngine = new RoutingEngine(routingSource, featureSource);
                routingEngine.RoutingAlgorithm.FindingRoute += new System.EventHandler<FindingRouteRoutingAlgorithmEventArgs>(RoutingAlgorithm_FindingRoute);
            }

            BaseShape avoidShape = BaseShape.CreateShapeFromWellKnownData(avoidWkt);
            featureSource.Open();
            Collection<Feature> allowFeatures = featureSource.SpatialQuery(avoidShape, QueryType.Within, ReturningColumnsType.NoColumns);
            allowFeatureIds = new Collection<string>();
            foreach (Feature item in allowFeatures)
            {
                allowFeatureIds.Add(item.Id);
            }

            RoutingResult result = routingEngine.GetRoute(startFeatureId, endFeatureId);
            string wkt = result.Route.GetWellKnownText();
            return wkt;
        }

        void RoutingAlgorithm_FindingRoute(object sender, FindingRouteRoutingAlgorithmEventArgs e)
        {
            Collection<string> beContainedFeatureIds = new Collection<string>();

            featureSource.Open();
            Collection<string> startPointAdjacentIds = e.RouteSegment.StartPointAdjacentIds;
            Collection<string> endPointAdjacentIds = e.RouteSegment.EndPointAdjacentIds;
            featureSource.Close();

            foreach (string id in startPointAdjacentIds)
            {
                if (!allowFeatureIds.Contains(id))
                {
                    beContainedFeatureIds.Add(id);
                }
            }
            foreach (string id in endPointAdjacentIds)
            {
                if (!allowFeatureIds.Contains(id))
                {
                    beContainedFeatureIds.Add(id);
                }
            }

            // Remove the ones that be contained in the avoidable area
            foreach (string id in beContainedFeatureIds)
            {
                if (e.RouteSegment.StartPointAdjacentIds.Contains(id))
                {
                    e.RouteSegment.StartPointAdjacentIds.Remove(id);
                }
                if (e.RouteSegment.EndPointAdjacentIds.Contains(id))
                {
                    e.RouteSegment.EndPointAdjacentIds.Remove(id);
                }
            }
        }
    }
}