﻿using System;
using System.Collections.ObjectModel;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.Routing;
using System.IO;
using System.Reflection;

namespace CSharp_Silverlight_HowDoISamples.Services
{
    // NOTE: If you change the class name "OptimizeRoadType" here, you must also update the reference to "OptimizeRoadType" in Web.config.
    public class OptimizeRoadType : IOptimizeRoadType
    {
        private static ShapeFileFeatureSource featureSource;
        private static RoutingSource routingSource;
        private static RoutingEngine routingEngine;
        private static string type;

        public OptimizeRoadType()
        {
            string excutePath = Path.GetDirectoryName(Assembly.GetAssembly(typeof(GeoDataGetting)).CodeBase);
            string appPath = excutePath.Substring(excutePath.IndexOf('\\') + 1, excutePath.LastIndexOf('\\') - excutePath.IndexOf('\\') - 1);

            featureSource = new ShapeFileFeatureSource(appPath + @"\App_Data\Austinstreets.shp");
            routingSource = new RtgRoutingSource(appPath + @"\App_Data\HighwayFirst.rtg");

            routingEngine = new RoutingEngine(routingSource, featureSource);
            routingEngine.RoutingAlgorithm.FindingRoute += new EventHandler<FindingRouteRoutingAlgorithmEventArgs>(RoutingAlgorithm_FindingRoute);
        }

        public string GetRoute(string startFeatureId, string endFeatureId, string routeType)
        {
            type = routeType;
            RoutingResult result = routingEngine.GetRoute(startFeatureId, endFeatureId);

            string wkt = result.Route.GetWellKnownText();
            return wkt;
        }

        void RoutingAlgorithm_FindingRoute(object sender, FindingRouteRoutingAlgorithmEventArgs e)
        {
            Collection<string> highways = new Collection<string>();
            Collection<string> adjacentIds = e.RouteSegment.StartPointAdjacentIds;
            foreach (string id in e.RouteSegment.EndPointAdjacentIds)
            {
                adjacentIds.Add(id);
            }
            foreach (string item in adjacentIds)
            {
                RouteSegment road = routingEngine.RoutingSource.GetRouteSegmentByFeatureId(item);
                if (road.RouteSegmentType == 2)
                {
                    highways.Add(item);
                }
            }

            if (type == "On Foot")
            {
                foreach (string item in highways)
                {
                    e.RouteSegment.StartPointAdjacentIds.Remove(item);
                    e.RouteSegment.EndPointAdjacentIds.Remove(item);
                }
            }

        }
    }
}
