﻿using System;
using System.Collections.ObjectModel;
using System.Globalization;
using System.IO;
using System.Reflection;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.Routing;

namespace CSharp_Silverlight_HowDoISamples.Services
{
    // NOTE: If you change the class name "RouteAroundRoadblocks" here, you must also update the reference to "RouteAroundRoadblocks" in Web.config.
    public class RouteAroundRoadblocks : IRouteAroundRoadblocks
    {
        private static ShapeFileFeatureSource featureSource;
        private static RoutingSource routingSource;
        private static RoutingEngine routingEngine;
        private static Collection<string> avoidableFeatureIds;
        private static Feature startFeature;
        private static Feature endFeature;

        public RouteAroundRoadblocks()
        {
            string excutePath = Path.GetDirectoryName(Assembly.GetAssembly(typeof(GeoDataGetting)).CodeBase);
            string appPath = excutePath.Substring(excutePath.IndexOf('\\') + 1, excutePath.LastIndexOf('\\') - excutePath.IndexOf('\\') - 1);

            featureSource = new ShapeFileFeatureSource(appPath + @"\App_Data\Austinstreets.shp");
            routingSource = new RtgRoutingSource(appPath + @"\App_Data\Austinstreets.rtg");

            avoidableFeatureIds = new Collection<string>();
            routingEngine = new RoutingEngine(routingSource, featureSource);
            routingEngine.RoutingAlgorithm.FindingRoute += new EventHandler<FindingRouteRoutingAlgorithmEventArgs>(RoutingAlgorithm_FindingRoute);
        }

        public string GetRoute(string startPoint, string endPoint, Collection<string> roadblocksPoints)
        {
            Collection<string> avoidablePoints = roadblocksPoints;
            featureSource.Open();
            PointShape start = ConvertStringToPointShape(startPoint);
            PointShape end = ConvertStringToPointShape(endPoint);
            if (startFeature == null)
            {
                startFeature = featureSource.GetFeaturesNearestTo(start, GeographyUnit.DecimalDegree, 1, ReturningColumnsType.NoColumns)[0];
            }
            if (endFeature == null)
            {
                endFeature = featureSource.GetFeaturesNearestTo(end, GeographyUnit.DecimalDegree, 1, ReturningColumnsType.NoColumns)[0];
            }
            foreach (string point in avoidablePoints)
            {
                PointShape pointShape = ConvertStringToPointShape(point);
                Collection<Feature> features = featureSource.GetFeaturesNearestTo(pointShape, GeographyUnit.DecimalDegree, 1, ReturningColumnsType.NoColumns);
                if (features.Count > 0 && features[0].Id != startFeature.Id && features[0].Id != endFeature.Id)
                {
                    avoidableFeatureIds.Add(features[0].Id);
                }
            }
            featureSource.Close();

            RoutingResult result = routingEngine.GetRoute(start, end);
            if (result.Route != null)
            {
                string wkt = result.Route.GetWellKnownText();
                return wkt;
            }
            return String.Empty;
        }

        void RoutingAlgorithm_FindingRoute(object sender, FindingRouteRoutingAlgorithmEventArgs e)
        {
            Collection<string> startPointAdjacentIds = e.RouteSegment.StartPointAdjacentIds;
            Collection<string> endPointAdjacentIds = e.RouteSegment.EndPointAdjacentIds;

            Collection<string> beContainedFeatureIds = new Collection<string>();
            foreach (string featureId in startPointAdjacentIds)
            {
                if (avoidableFeatureIds.Contains(featureId))
                {
                    beContainedFeatureIds.Add(featureId);
                }
            }
            foreach (string featureId in endPointAdjacentIds)
            {
                if (avoidableFeatureIds.Contains(featureId))
                {
                    beContainedFeatureIds.Add(featureId);
                }
            }

            // Remove the ones that be contained in the avoidable area
            foreach (string id in beContainedFeatureIds)
            {
                if (e.RouteSegment.StartPointAdjacentIds.Contains(id))
                {
                    e.RouteSegment.StartPointAdjacentIds.Remove(id);
                }
                if (e.RouteSegment.EndPointAdjacentIds.Contains(id))
                {
                    e.RouteSegment.EndPointAdjacentIds.Remove(id);
                }
            }
        }

        private PointShape ConvertStringToPointShape(string point)
        {
            string[] coordinates = point.Split(',');
            double x = double.Parse(coordinates[0], CultureInfo.InvariantCulture);
            double y = double.Parse(coordinates[1], CultureInfo.InvariantCulture);

            return new PointShape(x, y);
        }
    }
}
