﻿using System;
using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Threading;
using System.Windows.Browser;

namespace CSharp_Silverlight_HowDoISamples
{
    public partial class Banner : UserControl
    {
        private const string imageRootPath = "Theme/Banners/";
        private static Collection<ADSDetails> links;
        private DispatcherTimer timer;

        public Banner()
        {
            InitializeComponent();
        }

        private void LayoutRoot_Loaded(object sender, RoutedEventArgs e)
        {
            LoadUris();
            image.Source = new BitmapImage(links[0].ImageUri);
            image.Tag = links[0].TargetUri;

            timer = new DispatcherTimer();
            timer.Interval = TimeSpan.FromSeconds(4);
            timer.Tick += new EventHandler(timer_Tick);
            timer.Start();

        }

        private void timer_Tick(object sender, EventArgs e)
        {
            Random random = new Random(DateTime.Now.Millisecond);
            int index = random.Next(0, links.Count - 1);
            RandomImageAnimation(index);
            timer.Start();
        }

        private void RandomImageAnimation(int nextcurrent)
        {
            image.Source = new BitmapImage(links[nextcurrent].ImageUri);
            image.Tag = links[nextcurrent].TargetUri;
            Random sbRandom = new Random();
            switch (sbRandom.Next(1, 4))
            {
                case 1:
                    image.SetValue(OpacityMaskProperty, new SolidColorBrush(Color.FromArgb(0, 0, 0, 0)));
                    ImageStoryboard1.Begin();
                    break;
                case 2:
                    LinearGradientBrush lgb1 = new LinearGradientBrush();
                    GradientStop gs11 = new GradientStop();
                    GradientStop gs12 = new GradientStop();
                    gs11.SetValue(GradientStop.ColorProperty, Color.FromArgb(0, 0, 0, 0));
                    gs11.Offset = 0;
                    gs12.SetValue(GradientStop.ColorProperty, Color.FromArgb(255, 255, 255, 255));
                    gs12.Offset = 1;
                    lgb1.GradientStops.Add(gs11);
                    lgb1.GradientStops.Add(gs12);
                    image.OpacityMask = lgb1;
                    ImageStoryboard2.Begin();
                    break;
                case 3:
                    LinearGradientBrush lgb2 = new LinearGradientBrush();
                    GradientStop gs21 = new GradientStop();
                    GradientStop gs22 = new GradientStop();
                    gs21.SetValue(GradientStop.ColorProperty, Color.FromArgb(0, 0, 0, 0));
                    gs22.SetValue(GradientStop.ColorProperty, Color.FromArgb(255, 255, 255, 255));
                    gs22.Offset = 1;
                    lgb2.GradientStops.Add(gs21);
                    lgb2.GradientStops.Add(gs22);
                    lgb2.SetValue(LinearGradientBrush.StartPointProperty, new Point(-0.384999990463257,0.0469999983906746));
                    lgb2.SetValue(LinearGradientBrush.EndPointProperty, new Point(-0.388000011444092,-0.527999997138977));
                    image.OpacityMask = lgb2;
                    ImageStoryboard3.Begin();
                    break;
            }
        }

        private void LoadUris()
        {
            links = new Collection<ADSDetails>();

            ADSDetails adsCoder = new ADSDetails();
            adsCoder.TargetUri = new Uri("http://gis.thinkgeo.com/Products/GISComponentsforNETDevelopers/MapSuiteGeocodeUSA/tabid/159/Default.aspx");
            adsCoder.ImageUri = new Uri(imageRootPath + "bannerad_geocoder.png", UriKind.RelativeOrAbsolute);
            links.Add(adsCoder);

            ADSDetails adsDesktop = new ADSDetails();
            adsDesktop.TargetUri = new Uri("http://gis.thinkgeo.com/Products/GISComponentsforNETDevelopers/MapSuiteDesktopEdition/tabid/674/Default.aspx");
            adsDesktop.ImageUri = new Uri(imageRootPath + "bannerad_desktop.png", UriKind.RelativeOrAbsolute);
            links.Add(adsDesktop);

            ADSDetails adsForum = new ADSDetails();
            adsForum.TargetUri = new Uri("http://gis.thinkgeo.com/Support/DiscussionForums/tabid/143/Default.aspx");
            adsForum.ImageUri = new Uri(imageRootPath + "bannerad_forums.png", UriKind.RelativeOrAbsolute);
            links.Add(adsForum);

            ADSDetails adsWeb = new ADSDetails();
            adsWeb.TargetUri = new Uri("http://gis.thinkgeo.com/Products/GISComponentsforNETDevelopers/MapSuiteWebEdition/tabid/642/Default.aspx");
            adsWeb.ImageUri = new Uri(imageRootPath + "bannerad_web.png", UriKind.RelativeOrAbsolute);
            links.Add(adsWeb);

            ADSDetails adsCodeCommunity = new ADSDetails();
            adsCodeCommunity.TargetUri = new Uri("http://gis.thinkgeo.com/Admin/Vendors/BannerClickThrough.aspx?BannerId=42&VendorId=2&PortalId=1");
            adsCodeCommunity.ImageUri = new Uri(imageRootPath + "bannerad_codecommunity.png",UriKind.RelativeOrAbsolute);
            links.Add(adsCodeCommunity);

            ADSDetails adsPC = new ADSDetails();
            adsPC.TargetUri = new Uri("http://gis.thinkgeo.com/Products/GISComponentsforNETDevelopers/MapSuitePocketPC/tabid/163/Default.aspx");
            adsPC.ImageUri = new Uri(imageRootPath + "bannerad_pocketpc.png", UriKind.RelativeOrAbsolute);
            links.Add(adsPC);

            ADSDetails adsWorld = new ADSDetails();
            adsWorld.TargetUri = new Uri("http://gis.thinkgeo.com/Products/ExtensionsAddOnsforMapSuite/WorldMapKit/tabid/658/Default.aspx");
            adsWorld.ImageUri = new Uri(imageRootPath + "bannerad_worldmapkit.png", UriKind.RelativeOrAbsolute);
            links.Add(adsWorld);

            ADSDetails adsIcon = new ADSDetails();
            adsIcon.TargetUri = new Uri("http://gis.thinkgeo.com/Products/StandaloneMapDataIconLibraries/MapSuiteIconLibrary/SampleImages/tabid/537/Default.aspx");
            adsIcon.ImageUri = new Uri(imageRootPath + "bannerad_icon_library.png", UriKind.RelativeOrAbsolute);
            links.Add(adsIcon);

            ADSDetails offline = new ADSDetails();
            offline.TargetUri = new Uri("http://gis.thinkgeo.com");
            offline.ImageUri = new Uri(imageRootPath + "bannerad_offline.png", UriKind.RelativeOrAbsolute);
            links.Add(offline);
        }

        private void image_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            Image image = sender as Image;
            HtmlPage.Window.Navigate((Uri)image.Tag, "_blank");
        }
    }

    public class ADSDetails
    {
        public Uri ImageUri { get; set; }
        public Uri TargetUri { get; set; }
    }
}
