﻿using System;
using System.Linq;
using System.Windows;
using System.Windows.Browser;
using System.Windows.Controls;
using System.Windows.Input;
using System.Xml.Linq;
using ControlForDragPanel;
using ThinkGeo.MapSuite.SilverlightEdition;
using CSharp_Silverlight_HowDoISamples.Samples;

namespace CSharp_Silverlight_HowDoISamples
{
    public partial class MainPage : UserControl
    {
        private string selectedPage = "SimpleRouting";
        private Map map1;
        private DragPanel dragPanel;
        private UserControl currentSample;

        public MainPage()
        {
            InitializeComponent();

            Loaded += new RoutedEventHandler(MainPage_Loaded);
        }

        void MainPage_Loaded(object sender, RoutedEventArgs e)
        {
            string uri = HtmlPage.Document.DocumentUri.ToString();
            if (uri.EndsWith("HTML", StringComparison.OrdinalIgnoreCase))
            {
                HtmlPage.Window.Alert("Please set CSharp HowDoISamples.Web as start project. You can't use ServerLayerOverlay only in Html page.");
                return;
            }

            LoadTreeView();
            if ((((Microsoft.Windows.Controls.TreeViewItem)treeView.Items[0])).Items[0] != null)
            {
                Microsoft.Windows.Controls.TreeViewItem firstNode = (Microsoft.Windows.Controls.TreeViewItem)((Microsoft.Windows.Controls.TreeViewItem)treeView.Items[0]).Items[0];
                //Type type = typeof(SimpleRouting);
                UIElement element = GetControl(firstNode.Tag.ToString()) as UIElement;
                currentSample = (UserControl)element;
                dragPanel = currentSample.FindName("DescInfo") as DragPanel;
                ResetElementSize(currentSample);

                MapContainer.Children.Clear();
                MapContainer.Children.Add(element);

                pageScroll.ScrollToVerticalOffset(0);

                sampleTile.Text = "SimpleRouting";
                ((Microsoft.Windows.Controls.TreeViewItem)treeView.Items[0]).IsExpanded = true;
                firstNode.IsExpanded = true;
                double pageWidth = Convert.ToDouble(HtmlPage.Document.Body.GetProperty("scrollWidth"));
                bgSetting.Width = pageWidth - 250;
                btnSample.SetValue(Canvas.LeftProperty, pageWidth - 470);
                btnCSharpSource.SetValue(Canvas.LeftProperty, pageWidth - 380);
                firstNode.IsSelected = true;
            }
            Banners.Children.Add(new Banner());
        }

        private UserControl GetControl(string id)
        {
            UserControl control;
            switch (id)
            {
                case "GetShortestPathByCoordinates":
                    control = new GetShortestPathByCoordinates();
                    break;
                case "GetRoadInformationByRoadId":
                    control = new GetRoadInformationByRoadId();
                    break;
                case "RouteAvoidingCertainArea":
                    control = new RouteAvoidingCertainArea();
                    break;
                case "RouteOnlyInASpecifcArea":
                    control = new RouteOnlyInASpecifcArea();
                    break;
                case "UseDifferentAlgorithms":
                    control = new UseDifferentAlgorithms();
                    break;
                case "RoutingDirections":
                    control = new RoutingDirections();
                    break;
                case "ExtendingRoutingSource":
                    control = new ExtendingRoutingSource();
                    break;
                case "IntermediatePointsSupport":
                    control = new IntermediatePointsSupport();
                    break;
                case "ServiceAreaDefinition":
                    control = new ServiceAreaDefinition();
                    break;
                case "OptimizeRoadType":
                    control = new OptimizeRoadType();
                    break;
                case "OneWayRouting":
                    control = new OneWayRouting();
                    break;
                case "RoutingAroundRoadblocks":
                    control = new RoutingAroundRoadblocks();
                    break;
                case "TravelingSalesman":
                    control = new TravelingSalesman();
                    break;
                default:
                    control = new SimpleRouting();
                    break;
            }

            return control;
        }

        private void ResetElementSize(UserControl element)
        {
            if (element == null)
            {
                return;
            }

            double width = (double)HtmlPage.Document.Body.GetProperty("scrollWidth") - 255;
            double height = (double)HtmlPage.Document.Body.GetProperty("scrollHeight") - 130;
            element.Width = width;
            element.Height = height;
            Map map = element.FindName("Map1") as Map;
            if (map != null)
            {
                map.Width = width;
                map.Height = height;
            }

            DragPanel dragPanel = element.FindName("DescInfo") as DragPanel;
            if (dragPanel != null)
            {
                dragPanel.SetValue(Canvas.LeftProperty, width - 300);
                dragPanel.SetValue(Canvas.TopProperty, 6.0);
            }
        }

        private void LoadTreeView()
        {
            XDocument xmlFile = XDocument.Load("SampleList.xml");
            var samples = from c in xmlFile.Root.Elements()
                          select c;

            Microsoft.Windows.Controls.TreeViewItem sampleNode = new Microsoft.Windows.Controls.TreeViewItem();
            sampleNode.Header = "Samples";
            sampleNode.IsExpanded = true;
            sampleNode.IsSelected = true;

            foreach (XElement sample in samples)
            {
                Microsoft.Windows.Controls.TreeViewItem node = new Microsoft.Windows.Controls.TreeViewItem();
                node.Header = sample.Attribute("Name").Value;
                node.Tag = sample.Attribute("Id").Value;
                node.IsExpanded = false;
                sampleNode.Items.Add(node);
            }
            this.treeView.Items.Add(sampleNode);
        }

        private void treeView_SelectedItemChanged(object sender, RoutedPropertyChangedEventArgs<object> e)
        {
            var treeView = sender as Microsoft.Windows.Controls.TreeView;

            var tag = ((Microsoft.Windows.Controls.TreeViewItem)treeView.SelectedItem).Tag;
            if (tag != null)
            {
                selectedPage = tag.ToString();
            }

            LoadPageContent(treeView, tag);
        }

        private void LoadPageContent(Microsoft.Windows.Controls.TreeView treeView, object tag)
        {
            if (tag != null)
            {
                HtmlPage.Window.Eval("hide()");

                UIElement element = GetControl(tag.ToString());
                currentSample = (UserControl)element;

                ResetElementSize(currentSample);

                map1 = ((UserControl)element).FindName("Map1") as Map;
                dragPanel = ((UserControl)element).FindName("DescInfo") as DragPanel;
                MapContainer.Children.Clear();
                MapContainer.Children.Add(element);

                pageScroll.ScrollToVerticalOffset(0);

                if (treeView.SelectedItem != null)
                {
                    sampleTile.Text = ((Microsoft.Windows.Controls.TreeViewItem)treeView.SelectedItem).Header.ToString();
                }
                bgSetting.Width = content.ActualWidth - 2;
                btnSample.SetValue(Canvas.LeftProperty, content.ActualWidth - 220);
                btnCSharpSource.SetValue(Canvas.LeftProperty, content.ActualWidth - 130);
            }
        }

        private void BgChangeMouseDown(object sender, MouseButtonEventArgs e)
        {
            map1.Background = ((Canvas)sender).Background;
        }

        private void UserControl_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            ResetElementSize(currentSample);
        }

        private void btnSample_Click(object sender, RoutedEventArgs e)
        {
            LoadPageContent(treeView, selectedPage);
        }

        private void btnCSharpSource_Click(object sender, RoutedEventArgs e)
        {
            if (!string.IsNullOrEmpty(selectedPage))
            {
                string sourceCodePath = string.Format("SourceCode/{0}.xaml.html", selectedPage);
                HtmlPage.Window.Eval("show('" + sourceCodePath + "')");
            }
        }
    }
}
