﻿using System;
using System.Globalization;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media.Imaging;
using ThinkGeo.MapSuite.SilverlightCore;
using ThinkGeo.MapSuite.SilverlightEdition;

namespace CSharp_Silverlight_HowDoISamples
{
    public partial class GetShortestPathByCoordinates : UserControl
    {
        private SimpleMarkerOverlay markerOverlay;
        private InMemoryFeatureLayer routeLayer;

        public GetShortestPathByCoordinates()
        {
            InitializeComponent();
            this.Loaded += new RoutedEventHandler(GetShortestPathByCoordinates_Loaded);
        }

        void GetShortestPathByCoordinates_Loaded(object sender, RoutedEventArgs e)
        {
            Map1.MapUnit = GeographyUnit.DecimalDegree;

            WorldMapKitWmsSilverlightOverlay baseOverlay = new WorldMapKitWmsSilverlightOverlay();
            Map1.Overlays.Add(baseOverlay);

            markerOverlay = new SimpleMarkerOverlay();
            Map1.Overlays.Add("SimpleMarkerOverlay", markerOverlay);

            // Add the start and end flag
            string[] startPointString = startCoordinate.Text.Split(',');
            RenderMarker(new PointShape(double.Parse(startPointString[0], CultureInfo.InvariantCulture), double.Parse(startPointString[1], CultureInfo.InvariantCulture)), "start");
            string[] endPointString = endCoordinate.Text.Split(',');
            RenderMarker(new PointShape(double.Parse(endPointString[0], CultureInfo.InvariantCulture), double.Parse(endPointString[1], CultureInfo.InvariantCulture)), "end");

            routeLayer = new InMemoryFeatureLayer();
            routeLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = new LineStyle(new GeoPen(GeoColor.FromArgb(100, GeoColor.StandardColors.Purple), 5));
            routeLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle.OuterPen.LineJoin = DrawingLineJoin.Round;
            routeLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle.OuterPen.StartCap = GeoDashCap.Round;
            routeLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle.OuterPen.EndCap = GeoDashCap.Round;
            routeLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            LayerOverlay routeOverlay = new LayerOverlay();
            routeOverlay.Layers.Add("routeLayer", routeLayer);
            Map1.Overlays.Add("RouteOverlay", routeOverlay);

            Map1.CurrentExtent = new RectangleShape(-97.748300699833, 30.298843580812, -97.704999117497, 30.265927511782);
        }

        private void GetRoute_Click(object sender, RoutedEventArgs e)
        {
            GeoDataService.GeoDataGettingClient client = new CSharp_Silverlight_HowDoISamples.GeoDataService.GeoDataGettingClient();
            client.GetRouteByCoordinateCompleted += new EventHandler<CSharp_Silverlight_HowDoISamples.GeoDataService.GetRouteByCoordinateCompletedEventArgs>(client_GetRouteByCoordinateCompleted);
            client.GetRouteByCoordinateAsync(startCoordinate.Text, endCoordinate.Text);
        }

        void client_GetRouteByCoordinateCompleted(object sender, CSharp_Silverlight_HowDoISamples.GeoDataService.GetRouteByCoordinateCompletedEventArgs e)
        {
            if (e.Error == null)
            {
                string wkt = e.Result;
                LineShape route = LineShape.CreateShapeFromWellKnownData(wkt) as LineShape;

                routeLayer.InternalFeatures.Clear();
                routeLayer.InternalFeatures.Add(new Feature(route));
                Map1.Refresh();
            }
        }

        private void RenderMarker(PointShape pointShape, string renderTag)
        {
            Marker marker = new Marker(pointShape);
            if (renderTag == "start")
            {
                marker.ImageSource = new BitmapImage(new Uri("/theme/StartpointMarker.png", UriKind.RelativeOrAbsolute));
            }
            else if (renderTag == "end")
            {
                marker.ImageSource = new BitmapImage(new Uri("/theme/EndpointMarker.png", UriKind.RelativeOrAbsolute));
            }
            marker.ImageOffsetY = -10;
            markerOverlay.Markers.Add(marker);
            Map1.Refresh();
        }
    }
}
