﻿using System;
using System.Globalization;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media.Imaging;
using ThinkGeo.MapSuite.SilverlightCore;
using ThinkGeo.MapSuite.SilverlightEdition;

namespace CSharp_Silverlight_HowDoISamples.Samples
{
    public partial class UseDifferentAlgorithms : UserControl
    {
        private SimpleMarkerOverlay markerOverlay;
        private InMemoryFeatureLayer routeLayer;

        public UseDifferentAlgorithms()
        {
            InitializeComponent();
            this.Loaded += new RoutedEventHandler(UseDifferentAlgorithms_Loaded);
        }

        void UseDifferentAlgorithms_Loaded(object sender, RoutedEventArgs e)
        {
            Map1.MapUnit = GeographyUnit.DecimalDegree;

            WorldMapKitWmsSilverlightOverlay baseOverlay = new WorldMapKitWmsSilverlightOverlay();
            Map1.Overlays.Add(baseOverlay);

            markerOverlay = new SimpleMarkerOverlay();
            Map1.Overlays.Add("SimpleMarkerOverlay", markerOverlay);
            // Add the start and end flag
            GetStartPointByFeatureId(startFeautreId.Text);

            routeLayer = new InMemoryFeatureLayer();
            routeLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = new LineStyle(new GeoPen(GeoColor.FromArgb(100, GeoColor.StandardColors.Purple), 5));
            routeLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle.OuterPen.LineJoin = DrawingLineJoin.Round;
            routeLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle.OuterPen.StartCap = GeoDashCap.Round;
            routeLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle.OuterPen.EndCap = GeoDashCap.Round;
            routeLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            LayerOverlay routeOverlay = new LayerOverlay();
            routeOverlay.Layers.Add("routeLayer", routeLayer);
            Map1.Overlays.Add("RouteOverlay", routeOverlay);

            Map1.CurrentExtent = new RectangleShape(-97.763384, 30.299707, -97.712382, 30.259309);
        }

        private void cmbAlgorithm_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            GeoDataService.GeoDataGettingClient client = new CSharp_Silverlight_HowDoISamples.GeoDataService.GeoDataGettingClient();
            client.GetRouteCompleted += new EventHandler<CSharp_Silverlight_HowDoISamples.GeoDataService.GetRouteCompletedEventArgs>(client_GetRouteCompleted);

            string algorithm = String.Empty;
            if (cmbAlgorithm != null)
            {
                string selectedAlgorithm = ((ContentControl)(cmbAlgorithm.Items[cmbAlgorithm.SelectedIndex])).Content.ToString();
                switch (selectedAlgorithm)
                {
                    case "A*":
                        algorithm = "A*";
                        break;
                    case "Bidirectional":
                        algorithm = "Bidirectional";
                        break;
                    default:
                        break;
                }
            }
            client.GetRouteAsync("4226", "8094", algorithm);
        }

        void client_GetRouteCompleted(object sender, CSharp_Silverlight_HowDoISamples.GeoDataService.GetRouteCompletedEventArgs e)
        {
            if (e.Error == null)
            {
                string wkt = e.Result;
                LineShape route = LineShape.CreateShapeFromWellKnownData(wkt) as LineShape;

                routeLayer.InternalFeatures.Clear();
                routeLayer.InternalFeatures.Add(new Feature(route));
                Map1.Refresh();
            }
        }

        internal void GetStartPointByFeatureId(string featureId)
        {
            GeoDataService.GeoDataGettingClient client = new CSharp_Silverlight_HowDoISamples.GeoDataService.GeoDataGettingClient();
            client.GetPointShapeByIdCompleted += new EventHandler<CSharp_Silverlight_HowDoISamples.GeoDataService.GetPointShapeByIdCompletedEventArgs>(client_GetStartPointShapeByIdCompleted);
            client.GetPointShapeByIdAsync(featureId);
        }

        void client_GetStartPointShapeByIdCompleted(object sender, CSharp_Silverlight_HowDoISamples.GeoDataService.GetPointShapeByIdCompletedEventArgs e)
        {
            if (e.Error == null)
            {
                string[] positions = e.Result.Split(',');
                PointShape pointShape = new PointShape(double.Parse(positions[0], CultureInfo.InvariantCulture), double.Parse(positions[1], CultureInfo.InvariantCulture));

                RenderMarker(pointShape, "start");
                GetEndPointByFeatureId(endFeautreId.Text);
            }
        }

        internal void GetEndPointByFeatureId(string featureId)
        {
            GeoDataService.GeoDataGettingClient client = new CSharp_Silverlight_HowDoISamples.GeoDataService.GeoDataGettingClient();
            client.GetPointShapeByIdCompleted += new EventHandler<CSharp_Silverlight_HowDoISamples.GeoDataService.GetPointShapeByIdCompletedEventArgs>(client_GetEndPointShapeByIdCompleted);
            client.GetPointShapeByIdAsync(featureId);
        }

        void client_GetEndPointShapeByIdCompleted(object sender, CSharp_Silverlight_HowDoISamples.GeoDataService.GetPointShapeByIdCompletedEventArgs e)
        {
            if (e.Error == null)
            {
                string[] positions = e.Result.Split(',');
                PointShape pointShape = new PointShape(double.Parse(positions[0], CultureInfo.InvariantCulture), double.Parse(positions[1], CultureInfo.InvariantCulture));

                RenderMarker(pointShape, "end");
            }
        }

        private void RenderMarker(PointShape pointShape, string renderTag)
        {
            Marker marker = new Marker(pointShape);
            if (renderTag == "start")
            {
                marker.ImageSource = new BitmapImage(new Uri("/theme/StartpointMarker.png", UriKind.RelativeOrAbsolute));
            }
            else if (renderTag == "end")
            {
                marker.ImageSource = new BitmapImage(new Uri("/theme/EndpointMarker.png", UriKind.RelativeOrAbsolute));
            }
            marker.ImageOffsetY = -10;
            markerOverlay.Markers.Add(marker);
            Map1.Refresh();
        }
    }
}
